#include <AFMotor.h>  // Добавляем библиотеку для работы с шилдом
#include <Servo.h>    // Добавляем библиотеку сервопривода

#define TrigPIN A0  // Соединяем PIN0 с контактом TRIG датчика расстояния
#define EchoPIN A1  // Соединяем PIN1 с контактом ECHO датчика расстояния

#define DCMROFF 25  // Задаем разницу между двигателями
AF_DCMotor M1 (1, MOTOR12_64KHZ); // Первый мотор подлючаем к М1 и устанавливаем  на 64 кГц частота ШИМ
AF_DCMotor M2 (2, MOTOR12_64KHZ); // Второй мотор подлючаем к М2 и устанавливаем  на 64 кГц частота ШИМ
Servo SER1; // Создаём объект для управления сервомотором
int Search (void) {                   // Задаём настройки датчика расстояния
  float Duration = 0.0;               
  float CM = 0.0;                     
  digitalWrite (TrigPIN, LOW);        
  delayMicroseconds (2);              
  digitalWrite (TrigPIN, HIGH);       
  delayMicroseconds (10);             
  digitalWrite (TrigPIN, LOW);        
  Duration = pulseIn (EchoPIN, HIGH); 
  CM = (Duration/58.8);               
  return CM;                          
}
int RightDistance, LeftDistance;  //Расстояние с обоих сторон 
float Distance = 0.00;           
void setup () {               

  pinMode (TrigPIN, OUTPUT);  // Объявляем TrigPIN как выходной
  pinMode (EchoPIN, INPUT);   // Объявляем EchoPIN как входной

  SER1.attach (10);           // Присоединяет сервопривод на выводе 10
}
void loop () {                                    // Основной цикл.
  SER1.write (80);                                // Сообщает сервоприводу положение на 80 градусов (лицом вперед).
  delay (100);                                    // Задержка 0.1 сек
  Distance = Search ();                           // Измерение расстояния в сантиметрах
  if (Distance < 30) {                            // Если расстояние до препятствия меньше 30см 

    M1.setSpeed (100);                            // Задаём скорость двигателей 100
    M2.setSpeed (100);                            
    ChangePath ();                                // Если движение вперед заблокировано, изменяем направление
  }
  else if ((Distance >= 30) && (Distance < 60)) { // Если расстояние до препятствия от 30 до 60 см

    M1.setSpeed (150);                            // Задаём скорость двигателей 150
    M2.setSpeed (150);                            
    Forward ();                                   // Едем вперед
  }
  else if ((Distance >= 60) && (Distance < 90)) { // Если расстояние до препятствия от 60 до 90 см

    M1.setSpeed (200);                            // Задаём скорость двигателей 150
    M2.setSpeed (200);                            
    Forward ();                                   //Едем вперед
  }
  else {                                          // Если препятствие не может быть найдено в 90 см.

    M1.setSpeed (250);                            //Задаём скорость двигателей 150
    M2.setSpeed (250);                            
    Forward ();                                   // Едем вперед
  }
}
void ChangePath () {         // Цикл изменения пути
  Stop ();                   // Остановка робота
  Backward ();               // Робот едет назад
  Stop ();                   // Остановка робота
  SER1.write (12);           // Проверяем расстояние справа
  delay (500);               // Задержка 0.5 сек
  RightDistance = Search (); // Устанавливаем расстояние справа
  delay (500);               // Задержка 0.5 сек
  SER1.write (160);          // Проверяем расстояние слева
  delay (500);              // Задержка 0.5 сек
  LeftDistance = Search ();  // Устанавливаем расстояние слева
  delay (500);               // Задержка 0.5 сек
  SER1.write (80);           // Смотрим вперед
  delay (500);               // Задержка 0.5 сек
  CompareDistance ();        // Находим самое длинное расстояние
}
void CompareDistance () {                   // Цикл сравнения расстояний.
  if (RightDistance > LeftDistance) {       // Если справа меньше препятствий, робот поворачивается вправо
     TurnRight ();                          
    }
  else if (LeftDistance > RightDistance) {  // Если слева меньше препятствий, робот поворачивается влево
      TurnLeft ();                          
    }
  else {                                    // Если с обоих сторон препятствие, робот разворачивается
      TurnAround ();                        
    }
}
void Forward () {   // Цикл движения вперед
  M1.run (FORWARD); 
  M2.run (FORWARD); 
}
void Backward () {    // Цикл движения назад
  M1.run (BACKWARD);  
  M2.run (BACKWARD);  
  delay (1000);       
}
void TurnRight () {           // Цикл поворота вправо
  M1.run (BACKWARD);        
  M2.run (FORWARD);          
  M1.setSpeed (100+DCMROFF);  
  delay (700);             
}
void TurnLeft () {            // Цикл поворота влево
  M1.run (FORWARD);          
  M2.run (BACKWARD);         
  M2.setSpeed (100+DCMROFF);  
  delay (700);                
}
void TurnAround () {          // Цикл разворота
  M1.run (FORWARD);           
  M2.run (BACKWARD);          
  M2.setSpeed (100+DCMROFF);  
  delay (2000);               
}
void Stop () {      // Цикл остановки
  M1.run (RELEASE); 
  M2.run (RELEASE); 
  delay (100);     
}
